﻿/*
 *  Copyright (c) 2013-2015, Cureos AB.
 *  All rights reserved.
 *  http://www.cureos.com
 *
 *	This file is part of Shim.Drawing.
 *
 *  Shim.Drawing is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Shim.Drawing is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Shim.Drawing.  If not, see <http://www.gnu.org/licenses/>.
 */

using System.Drawing.Imaging;

namespace System.Drawing
{
    using System.IO;

    public abstract class Image
    {
        #region PROPERTIES

        public abstract PixelFormat PixelFormat { get; }

        public abstract int Width { get; }

        public abstract int Height { get; }

        public abstract Size Size { get; }

        internal abstract ColorPalette Palette { get; set; }

        #endregion

        #region METHODS

        public static Image FromStream(Stream stream)
        {
            return Bitmap.Create(stream);
        }

        public abstract void Save(Stream stream, ImageFormat format);

        public abstract void Save(string filename, ImageCodecInfo encoder, EncoderParameters encoderParams);

        internal static int GetPixelFormatSize(PixelFormat pixelFormat)
        {
            switch (pixelFormat)
            {
                case PixelFormat.Format1bppIndexed:
                    return 1;
                case PixelFormat.Format4bppIndexed:
                    return 4;
                case PixelFormat.Format8bppIndexed:
                    return 8;
                case PixelFormat.Format16bppGrayScale:
                    return 16;
                case PixelFormat.Format24bppRgb:
                    return 24;
                case PixelFormat.Format32bppRgb:
                case PixelFormat.Format32bppArgb:
                case PixelFormat.Format32bppPArgb:
                    return 32;
                case PixelFormat.Format48bppRgb:
                    return 48;
                case PixelFormat.Format64bppArgb:
                case PixelFormat.Format64bppPArgb:
                    return 64;
                default:
                    throw new ArgumentOutOfRangeException("pixelFormat");
            }
        }
        
        #endregion
    }
}